library(tm)
library(wordcloud)
library(memoise)
library(topicmodels)


stopwordsCustom <- paste(c('can', 'say','one','way','use',
	'also','howev','tell','will',
	'much','need','take','tend','even',
	'like','particular','rather','said',
	'get','well','make','ask','come','end',
	'first','two','help','often','may',
	'might','see','someth','thing','point',
	'post','look','right','now','think','\'ve ',
	'\'re ','anoth','put','set','new','good',
	'want','sure','kind','larg','yes,','day','etc',
	'quit','sinc','attempt','lack','seen','awar',
	'littl','ever','moreov','though','found','abl',
	'enough','far','earli','away','achiev','draw',
	'last','never','brief','bit','entir','brief',
	'great','lot'),collapse=",")

#################################################
getInputData = memoise(function(fn)
{
	inputData=read.csv(fn,header=TRUE,stringsAsFactors=FALSE)
	return(inputData)
})

##################################################
stemDocProcessing = function(inputData,myStopwordLine)
{
	myStopwordsOrg=unlist(strsplit(myStopwordLine,","))
	myStopwords=stemDocument(unlist(strsplit(myStopwordLine,",")))
	
	docsOrg = Corpus(VectorSource(as.character(inputData$Content)))
	docs <-tm_map(docsOrg,content_transformer(tolower))
	docs <- tm_map(docs, removeWords, myStopwordsOrg)
	
	toSpace <- content_transformer(function(x, pattern) { return (gsub(pattern, ' ', x))})
	docs <- tm_map(docs, toSpace, '-')
	docs <- tm_map(docs, toSpace, '’')
	docs <- tm_map(docs, toSpace, '‘')
	docs <- tm_map(docs, toSpace, '•')
	docs <- tm_map(docs, toSpace, "'")
  
	#remove punctuation
	docs <- tm_map(docs, removePunctuation)
	#Strip digits
	docs <- tm_map(docs, removeNumbers)
	#remove stopwords
	docs <- tm_map(docs, removeWords, stopwords('english'))
	#remove whitespace
	docs <- tm_map(docs, stripWhitespace)
	#Stem document
	docsStemmed <- tm_map(docs,stemDocument)
  
  
	#fix up 1) differences between us and aussie english 2) general errors
	 docsStemmed <- tm_map(docsStemmed, content_transformer(gsub), pattern = 'nize ', replacement = 'ise ')
	 docsStemmed <- tm_map(docsStemmed, content_transformer(gsub), pattern = 'organis', replacement = 'organ')
	 docsStemmed <- tm_map(docsStemmed, content_transformer(gsub), pattern = 'andgovern', replacement = 'govern')
	 docsStemmed <- tm_map(docsStemmed, content_transformer(gsub), pattern = 'inenterpris', replacement = 'enterpris')
	 docsStemmed <- tm_map(docsStemmed, content_transformer(gsub),pattern = 'team-', replacement = 'team')
	#define and eliminate all custom stopwords
	docsStemmed <- tm_map(docsStemmed, removeWords, myStopwords)
	
	return(docsStemmed)
}



#################################################
getTermMatrixFromCSV = memoise(function(fn,myStopwordLine) 
{
	# Careful not to let just any name slip in here; a
	# malicious user could manipulate this value.
	#if (!(fn %in% filenames))
	#stop("Unknown filename")\
	inputData <- read.csv(fn,header=TRUE,stringsAsFactors=FALSE)
	docsOrg = Corpus(VectorSource(as.character(inputData$Content)))
	docs <-tm_map(docsOrg,content_transformer(tolower))
	
	
	docsStemmed=stemDocProcessing(inputData,myStopwordLine)
	
	myDTM = TermDocumentMatrix(docsStemmed,control = list(minWordLength = 1))
	rownames(myDTM) = gsub("[aeyuio]+$","",rownames(myDTM))
	rownames(myDTM)= stemCompletion(rownames(myDTM),docs,"prevalent")
	
	
	m = as.matrix(myDTM)
	sort(rowSums(m), decreasing = TRUE)
  
})


#################################################
getTermMatrixFromVector = memoise(function(inputData,myStopwordLine) 
{
	# Careful not to let just any name slip in here; a
	# malicious user could manipulate this value.
	#if (!(fn %in% filenames))
	#stop("Unknown filename")\
	#inputData <- read.csv(fn,header=TRUE,stringsAsFactors=FALSE)
	docsOrg = Corpus(VectorSource(as.character(inputData$Content)))
	docs <-tm_map(docsOrg,content_transformer(tolower))
	
	
	docsStemmed=stemDocProcessing(inputData,myStopwordLine)
	
	myDTM = TermDocumentMatrix(docsStemmed,control = list(minWordLength = 1))
	rownames(myDTM) = gsub("[aeyuio]+$","",rownames(myDTM))
	rownames(myDTM)= stemCompletion(rownames(myDTM),docs,"prevalent")
	
	
	m = as.matrix(myDTM)
	sort(rowSums(m), decreasing = TRUE)
  
})

#################################################
performLDAWithWordcloud = memoise(function(inputData,topicNum,minFreq,maxWord,inputBasename,myStopwordLine)
{
	#inputData=read.csv(fn,header=TRUE,stringsAsFactors=FALSE)
	docsStemmed=stemDocProcessing(inputData,myStopwordLine)
	
	#Create document-term matrix
	dtm <- DocumentTermMatrix(docsStemmed)
	#convert rownames to filenames
	rownames(dtm) <- inputData$ID
	
	
	#### Topic modeling
	#################################################################
	# For topic modelling
	#Set parameters for Gibbs sampling
	burnin <- 4000
	iter <- 2000
	thin <- 500
	seed <-list(2003,5,63,100001,765)
	nstart <- 5
	best <- TRUE
	#Number of topics
	k <- topicNum

	#Run LDA using Gibbs sampling
	## Latent Dirichlet Allocation
	## Predefine k which is the no. of topics to classify the document into
	## For each word in every document, randomly assign a topic number to the word
	## With this random assignment, you already can specify a document is how many percent of each topic
	## To improve the assignment of topics to words, iterate through the following
	## For each word in document, compute
	##    (i) p(topic t/doucment d) = proportion of words in doucment d that are currently assigned to topic t
	##    (ii) p(word w/topic t) = proportion of assignments to topic t over all documents that come from this word w
	##    Reassign w a new topic t with highest p(topic t | document d) * p(word w | topic t)ldaOut <-LDA(dtm,k, method='Gibbs', control=list(nstart=nstart, seed = seed, best=best, burnin = burnin, iter = iter, thin=thin))
	ldaOut <-LDA(dtm,k, method="Gibbs", control=list(nstart=nstart, seed = seed, best=best, burnin = burnin, iter = iter, thin=thin))
	ldaOut.topics <- as.matrix(topics(ldaOut))
	ldaOutDf=data.frame(ID=rownames(ldaOut.topics),Topics=as.numeric(ldaOut.topics[,1]))
	resultDir = sprintf('output/%s',inputBasename)
	dir.create(resultDir,showWarnings=FALSE)
	
	
	##### Create high freq words for each topic
	ldaTopicWords=as.character()
	wordcloudDir=sprintf('%s/%s_wordcloud_%d',resultDir,inputBasename,topicNum)
	dir.create(wordcloudDir,showWarnings=FALSE)
	for (tpNo in 1:topicNum)
	{
		wcFn=sprintf("%s/topic_%d.jpg",wordcloudDir,tpNo)
		jpeg(wcFn)
		flist=ldaOutDf$ID[which(ldaOutDf$Topics==tpNo)]
		inputContent=as.character(inputData$Content[which(inputData$ID %in% flist)])
		v=getTermMatrixFromVector(inputData[which(inputData$ID %in% flist),],myStopwordLine)
		wordcloud(words = names(v), freq = v, min.freq = minFreq, max.words=maxWord, random.order=FALSE, rot.per=0.35, colors=brewer.pal(8, "Dark2"))
		dev.off()
		ldaTopicWords=c(ldaTopicWords,paste(names(v[1:10]),collapse=","))
		
	}
	
	ldaTopicWordsDf=data.frame(Topics=seq(1,topicNum),Keywords=ldaTopicWords)
	ldaOutDf=merge(ldaOutDf,ldaTopicWordsDf,by=c('Topics'),all.x=TRUE)
	
	ldaOutDf_withContent=merge(ldaOutDf,inputData[,c('ID','Content')],by=c('ID'),all.x=TRUE)
	write.csv(ldaOutDf_withContent,sprintf('%s/%s_ldaOutDf.csv',resultDir,inputBasename))
	
	
	return(ldaOutDf)

})



###################################################################################################
get_plot_output_list <- function(ldaOutput,inputData,minFreq,maxWord,myStopwordLine) 
{
	
  # Insert plot output objects the list
  #inputData=read.csv(fn,header=TRUE,stringsAsFactors=FALSE)
  uniqueTopics=unique(ldaOutput$Topics)
  plot_output_list <- lapply(1:length(uniqueTopics), function(i) {  
    plotname <- paste("plot", i, sep="")
    plot_output_object <- plotOutput(plotname, height = 280, width = 250)
    plot_output_object <- renderPlot({
		flist=ldaOutput$ID[which(ldaOutput$Topics==uniqueTopics[i])]
		v=getTermMatrixFromVector(inputData[which(inputData$ID %in% flist),],myStopwordLine)
		wordcloud(words = names(v), freq = v, min.freq = minFreq, max.words=maxWord, random.order=FALSE, rot.per=0.35, colors=brewer.pal(8, "Dark2"))
		#plot(seq(1:10),seq(1:10),main=sprintf("%d %s",length(v),names(v)[1]))
    })
  })

  #do.call(tagList, plot_output_list) # needed to display properly.

  #return(plot_output_list)
}


###################################################################################################
get_plot_output_list_test<- function(input_n)
{
		plot_output_list <- lapply(1:input_n, function(i) {  
			plotname <- paste("plot", i, sep="")
			plot_output_object <- plotOutput(plotname, height = 280, width = 250)
			plot_output_object <- renderPlot({
				plot(seq(1:10),seq(1:10),main='abc')
			})
  })
}


###################################################################################################
getStopwords = function(stopwords)
{
	stopwordArry=unlist(strsplit(stopwords,","))
	stopwordList=data.frame(stopwords=stopwordArry,stopwordsStem=stemDocument(stopwordArry))
	return(stopwordList)
}
